@@ -1,3 +1,7 @@
+#' @include table.R
+#' @include hidden.R
+NULL
+
 #' DBIConnection class
 #'
 #' This virtual class encapsulates the connection to a DBMS, and it provides
@@ -34,7 +38,8 @@ setMethod("show", "DBIConnection", function(object) {
   # RPostgreSQL)
   tryCatch(
     show_connection(object),
-    error = function(e) NULL)
+    error = function(e) NULL
+  )
   invisible(NULL)
 })
 
@@ -50,6 +55,9 @@ show_connection <- function(object) {
 #' This closes the connection, discards all pending work, and frees
 #' resources (e.g., memory, sockets).
 #'
+#' @template methods
+#' @templateVar method_name dbDisconnect
+#'
 #' @inherit DBItest::spec_connection_disconnect return
 #' @inheritSection DBItest::spec_connection_disconnect Specification
 #'
@@ -83,8 +91,14 @@ setGeneric("dbDisconnect",
 #' and transfer them piecemeal to R, others may transfer all the data to the
 #' client -- but not necessarily to the memory that R manages. See individual
 #' drivers' `dbSendQuery()` documentation for details.
+#'
+#' @template methods
+#' @templateVar method_name dbSendQuery
+#'
 #' @inherit DBItest::spec_result_send_query return
+#' @inheritSection DBItest::spec_result_send_query Additional arguments
 #' @inheritSection DBItest::spec_result_send_query Specification
+#' @inheritSection DBItest::spec_result_send_query Specification for the `immediate` argument
 #'
 #' @inheritParams dbGetQuery
 #' @param statement a character string containing SQL.
@@ -95,7 +109,24 @@ setGeneric("dbDisconnect",
 #' con <- dbConnect(RSQLite::SQLite(), ":memory:")
 #'
 #' dbWriteTable(con, "mtcars", mtcars)
-#' rs <- dbSendQuery(con, "SELECT * FROM mtcars WHERE cyl = 4;")
+#' rs <- dbSendQuery(con, "SELECT * FROM mtcars WHERE cyl = 4")
+#' dbFetch(rs)
+#' dbClearResult(rs)
+#'
+#' # Pass one set of values with the param argument:
+#' rs <- dbSendQuery(
+#'   con,
+#'   "SELECT * FROM mtcars WHERE cyl = ?",
+#'   params = list(4L)
+#' )
+#' dbFetch(rs)
+#' dbClearResult(rs)
+#'
+#' # Pass multiple sets of values with dbBind():
+#' rs <- dbSendQuery(con, "SELECT * FROM mtcars WHERE cyl = ?")
+#' dbBind(rs, list(6L))
+#' dbFetch(rs)
+#' dbBind(rs, list(8L))
 #' dbFetch(rs)
 #' dbClearResult(rs)
 #'
@@ -120,8 +151,13 @@ setGeneric("dbSendQuery",
 #' forwards to [dbSendQuery()], to support backends that only
 #' implement the latter.
 #'
+#' @template methods
+#' @templateVar method_name dbSendStatement
+#'
 #' @inherit DBItest::spec_result_send_statement return
+#' @inheritSection DBItest::spec_result_send_statement Additional arguments
 #' @inheritSection DBItest::spec_result_send_statement Specification
+#' @inheritSection DBItest::spec_result_send_statement Specification for the `immediate` argument
 #'
 #' @inheritParams dbGetQuery
 #' @param statement a character string containing SQL.
@@ -132,13 +168,34 @@ setGeneric("dbSendQuery",
 #' con <- dbConnect(RSQLite::SQLite(), ":memory:")
 #'
 #' dbWriteTable(con, "cars", head(cars, 3))
-#' rs <- dbSendStatement(con,
-#'   "INSERT INTO cars (speed, dist) VALUES (1, 1), (2, 2), (3, 3);")
+#'
+#' rs <- dbSendStatement(
+#'   con,
+#'   "INSERT INTO cars (speed, dist) VALUES (1, 1), (2, 2), (3, 3)"
+#' )
 #' dbHasCompleted(rs)
 #' dbGetRowsAffected(rs)
 #' dbClearResult(rs)
 #' dbReadTable(con, "cars")   # there are now 6 rows
 #'
+#' # Pass one set of values directly using the param argument:
+#' rs <- dbSendStatement(
+#'   con,
+#'   "INSERT INTO cars (speed, dist) VALUES (?, ?)",
+#'   params = list(4L, 5L)
+#' )
+#' dbClearResult(rs)
+#'
+#' # Pass multiple sets of values using dbBind():
+#' rs <- dbSendStatement(
+#'   con,
+#'   "INSERT INTO cars (speed, dist) VALUES (?, ?)"
+#' )
+#' dbBind(rs, list(5:6, 6:7))
+#' dbBind(rs, list(7L, 8L))
+#' dbClearResult(rs)
+#' dbReadTable(con, "cars")   # there are now 10 rows
+#'
 #' dbDisconnect(con)
 #' @export
 setGeneric("dbSendStatement",
@@ -163,13 +220,25 @@ setMethod(
 #' [dbSendQuery()], then [dbFetch()], ensuring that
 #' the result is always free-d by [dbClearResult()].
 #'
-#' This method is for `SELECT` queries only.  Some backends may
+#' This method is for `SELECT` queries only
+#' (incl. other SQL statements that return a `SELECT`-alike result,
+#'  e. g. execution of a stored procedure).
+#'
+#' To execute a stored procedure that does not return a result set,
+#' use [dbExecute()].
+#'
+#' Some backends may
 #' support data manipulation statements through this method for compatibility
 #' reasons.  However, callers are strongly advised to use
 #' [dbExecute()] for data manipulation statements.
 #'
+#' @template methods
+#' @templateVar method_name dbGetQuery
+#'
 #' @inherit DBItest::spec_result_get_query return
+#' @inheritSection DBItest::spec_result_get_query Additional arguments
 #' @inheritSection DBItest::spec_result_get_query Specification
+#' @inheritSection DBItest::spec_result_get_query Specification for the `immediate` argument
 #'
 #' @section Implementation notes:
 #' Subclasses should override this method only if they provide some sort of
@@ -187,6 +256,17 @@ setMethod(
 #'
 #' dbWriteTable(con, "mtcars", mtcars)
 #' dbGetQuery(con, "SELECT * FROM mtcars")
+#' dbGetQuery(con, "SELECT * FROM mtcars", n = 6)
+#'
+#' # Pass values using the param argument:
+#' # (This query runs eight times, once for each different
+#' # parameter. The resulting rows are combined into a single
+#' # data frame.)
+#' dbGetQuery(
+#'   con,
+#'   "SELECT COUNT(*) FROM mtcars WHERE cyl = ?",
+#'   params = list(1:8)
+#' )
 #'
 #' dbDisconnect(con)
 setGeneric("dbGetQuery",
@@ -213,11 +293,21 @@ setMethod("dbGetQuery", signature("DBIConnection", "character"),
 #' [dbSendStatement()], then [dbGetRowsAffected()], ensuring that
 #' the result is always free-d by [dbClearResult()].
 #'
+#' You can also use `dbExecute()` to call a stored procedure
+#' that performs data manipulation or other actions that do not return a result set.
+#' To execute a stored procedure that returns a result set use [dbGetQuery()] instead.
+#'
+#' @template methods
+#' @templateVar method_name dbExecute
+#'
 #' @section Implementation notes:
 #' Subclasses should override this method only if they provide some sort of
 #' performance optimization.
 #'
 #' @inherit DBItest::spec_result_execute return
+#' @inheritSection DBItest::spec_result_execute Additional arguments
+#' @inheritSection DBItest::spec_result_execute Specification
+#' @inheritSection DBItest::spec_result_execute Specification for the `immediate` argument
 #'
 #' @inheritParams dbGetQuery
 #' @param statement a character string containing SQL.
@@ -229,10 +319,20 @@ setMethod("dbGetQuery", signature("DBIConnection", "character"),
 #'
 #' dbWriteTable(con, "cars", head(cars, 3))
 #' dbReadTable(con, "cars")   # there are 3 rows
-#' dbExecute(con,
-#'   "INSERT INTO cars (speed, dist) VALUES (1, 1), (2, 2), (3, 3);")
+#' dbExecute(
+#'   con,
+#'   "INSERT INTO cars (speed, dist) VALUES (1, 1), (2, 2), (3, 3)"
+#' )
 #' dbReadTable(con, "cars")   # there are now 6 rows
 #'
+#' # Pass values using the param argument:
+#' dbExecute(
+#'   con,
+#'   "INSERT INTO cars (speed, dist) VALUES (?, ?)",
+#'   params = list(4:7, 5:8)
+#' )
+#' dbReadTable(con, "cars")   # there are now 10 rows
+#'
 #' dbDisconnect(con)
 setGeneric("dbExecute",
   def = function(conn, statement, ...) standardGeneric("dbExecute")
@@ -251,25 +351,32 @@ setMethod(
 
 #' Get DBMS exceptions
 #'
+#' DEPRECATED. Backends should use R's condition system to signal errors and
+#' warnings.
+#'
 #' @inheritParams dbGetQuery
 #' @family DBIConnection generics
 #' @return a list with elements `errorNum` (an integer error number) and
 #'   `errorMsg` (a character string) describing the last error in the
 #'   connection `conn`.
 #' @export
+#' @keywords internal
 setGeneric("dbGetException",
   def = function(conn, ...) standardGeneric("dbGetException")
 )
 
 #' A list of all pending results
 #'
-#' List of [DBIResult-class] objects currently active on the connection.
+#' DEPRECATED. DBI currenty supports only one open result set per connection,
+#' you need to keep track of the result sets you open if you need this
+#' functionality.
 #'
 #' @inheritParams dbGetQuery
 #' @family DBIConnection generics
 #' @return a list. If no results are active, an empty list. If only
 #'   a single result is active, a list with one element.
 #' @export
+#' @keywords internal
 setGeneric("dbListResults",
   def = function(conn, ...) standardGeneric("dbListResults")
 )
@@ -278,7 +385,10 @@ setGeneric("dbListResults",
 #'
 #' @inheritParams dbGetQuery
 #' @param name a character string with the name of the remote table.
-#' @return a character vector
+#'
+#' @inherit DBItest::spec_sql_list_fields return
+#' @inheritSection DBItest::spec_sql_list_fields Specification
+#'
 #' @family DBIConnection generics
 #' @seealso [dbColumnInfo()] to get the type of the fields.
 #' @export
@@ -294,14 +404,47 @@ setGeneric("dbListFields",
   valueClass = "character"
 )
 
+#' @rdname hidden_aliases
+#' @export
+setMethod("dbListFields", signature("DBIConnection", "character"),
+  function(conn, name, ...) {
+    list_fields(conn, name)
+  }
+)
+
+#' @rdname hidden_aliases
+#' @export
+setMethod("dbListFields", signature("DBIConnection", "Id"),
+  function(conn, name, ...) {
+    list_fields(conn, name)
+  }
+)
+
+list_fields <- function(conn, name) {
+  rs <- dbSendQuery(
+    conn,
+    paste(
+      "SELECT * FROM ",
+      dbQuoteIdentifier(conn, name),
+      "LIMIT 0"
+    )
+  )
+  on.exit(dbClearResult(rs))
+
+  names(dbFetch(rs, n = 0, row.names = FALSE))
+}
+
 #' List remote tables
 #'
 #' Returns the unquoted names of remote tables accessible through this
 #' connection.
-#' This should, where possible, include temporary tables, and views.
+#' This should include views and temporary objects, but not all database backends
+#' (in particular \pkg{RMariaDB} and \pkg{RMySQL}) support this.
+#'
+#' @template methods
+#' @templateVar method_name dbListTables
 #'
 #' @inherit DBItest::spec_sql_list_tables return
-#' @inheritSection DBItest::spec_sql_list_tables Additional arguments
 #'
 #' @inheritParams dbGetQuery
 #' @family DBIConnection generics
@@ -319,12 +462,68 @@ setGeneric("dbListTables",
   valueClass = "character"
 )
 
+#' List remote objects
+#'
+#' Returns the names of remote objects accessible through this connection
+#' as a data frame.
+#' This should include temporary objects, but not all database backends
+#' (in particular \pkg{RMariaDB} and \pkg{RMySQL}) support this.
+#' Compared to [dbListTables()], this method also enumerates tables and views
+#' in schemas, and returns fully qualified identifiers to access these objects.
+#' This allows exploration of all database objects available to the current
+#' user, including those that can only be accessed by giving the full
+#' namespace.
+#'
+#' @template methods
+#' @templateVar method_name dbListObjects
+#'
+#' @inherit DBItest::spec_sql_list_objects return
+#' @inheritSection DBItest::spec_sql_list_objects Additional arguments
+#' @inheritSection DBItest::spec_sql_list_objects Specification
+#'
+#' @inheritParams dbGetQuery
+#' @param prefix A fully qualified path in the database's namespace, or `NULL`.
+#'   This argument will be processed with [dbUnquoteIdentifier()].
+#'   If given the method will return all objects accessible through this prefix.
+#' @family DBIConnection generics
+#' @export
+#' @examples
+#' con <- dbConnect(RSQLite::SQLite(), ":memory:")
+#'
+#' dbListObjects(con)
+#' dbWriteTable(con, "mtcars", mtcars)
+#' dbListObjects(con)
+#'
+#' dbDisconnect(con)
+setGeneric("dbListObjects",
+  def = function(conn, prefix = NULL, ...) standardGeneric("dbListObjects"),
+  valueClass = "data.frame"
+)
+
+#' @rdname hidden_aliases
+#' @export
+setMethod("dbListObjects", signature("DBIConnection", "ANY"),
+  function(conn, prefix = NULL, ...) {
+    names <- dbListTables(conn)
+    tables <- lapply(names, function(x) Id(table = x))
+    ret <- data.frame(
+      table = I(unname(tables)),
+      stringsAsFactors = FALSE
+    )
+    ret$is_prefix <- rep_len(FALSE, nrow(ret))
+    ret
+  }
+)
+
 #' Copy data frames from database tables
 #'
 #' Reads a database table to a data frame, optionally converting
 #' a column to row names and converting the column names to valid
 #' R identifiers.
 #'
+#' @template methods
+#' @templateVar method_name dbReadTable
+#'
 #' @inherit DBItest::spec_sql_read_table return
 #' @inheritSection DBItest::spec_sql_read_table Additional arguments
 #' @inheritSection DBItest::spec_sql_read_table Specification
@@ -348,14 +547,16 @@ setGeneric("dbReadTable",
 
 #' @rdname hidden_aliases
 #' @export
-setMethod("dbReadTable", c("DBIConnection", "character"),
-  function(conn, name, ..., row.names = NA, check.names = TRUE) {
+setMethod("dbReadTable", signature("DBIConnection", "character"),
+  function(conn, name, ..., row.names = FALSE, check.names = TRUE) {
     sql_name <- dbQuoteIdentifier(conn, x = name, ...)
     if (length(sql_name) != 1L) {
       stop("Invalid name: ", format(name), call. = FALSE)
     }
-    stopifnot(length(row.names) == 1L)
-    stopifnot(is.null(row.names) || is.logical(row.names) || is.character(row.names))
+    if (!is.null(row.names)) {
+      stopifnot(length(row.names) == 1L)
+      stopifnot(is.logical(row.names) || is.character(row.names))
+    }
     stopifnot(length(check.names) == 1L)
     stopifnot(is.logical(check.names))
     stopifnot(!is.na(check.names))
@@ -369,10 +570,22 @@ setMethod("dbReadTable", c("DBIConnection", "character"),
   }
 )
 
+#' @rdname hidden_aliases
+#' @export
+setMethod("dbReadTable", signature("DBIConnection", "Id"),
+  function(conn, name, ...) {
+    dbReadTable(conn, dbQuoteIdentifier(conn, name), ...)
+  }
+)
+
 #' Copy data frames to database tables
 #'
 #' Writes, overwrites or appends a data frame to a database table, optionally
 #' converting row names to a column and specifying SQL data types for fields.
+#' New code should prefer [dbCreateTable()] and [dbAppendTable()].
+#'
+#' @template methods
+#' @templateVar method_name dbWriteTable
 #'
 #' @inherit DBItest::spec_sql_write_table return
 #' @inheritSection DBItest::spec_sql_write_table Additional arguments
@@ -404,12 +617,22 @@ setGeneric("dbWriteTable",
   def = function(conn, name, value, ...) standardGeneric("dbWriteTable")
 )
 
+#' @rdname hidden_aliases
+#' @export
+setMethod("dbWriteTable", signature("DBIConnection", "Id", "ANY"),
+  function(conn, name, value, ...) {
+    dbWriteTable(conn, dbQuoteIdentifier(conn, name), value, ...)
+  }
+)
+
 #' Does a table exist?
 #'
 #' Returns if a table given by name exists in the database.
 #'
+#' @template methods
+#' @templateVar method_name dbExistsTable
+#'
 #' @inherit DBItest::spec_sql_exists_table return
-#' @inheritSection DBItest::spec_sql_exists_table Additional arguments
 #' @inheritSection DBItest::spec_sql_exists_table Specification
 #'
 #' @inheritParams dbGetQuery
@@ -429,12 +652,24 @@ setGeneric("dbExistsTable",
   valueClass = "logical"
 )
 
+#' @rdname hidden_aliases
+#' @export
+setMethod("dbExistsTable", signature("DBIConnection", "Id"),
+  function(conn, name, ...) {
+    dbExistsTable(conn, dbQuoteIdentifier(conn, name), ...)
+  }
+)
+
 #' Remove a table from the database
 #'
 #' Remove a remote table (e.g., created by [dbWriteTable()])
 #' from the database.
 #'
+#' @template methods
+#' @templateVar method_name dbRemoveTable
+#'
 #' @inherit DBItest::spec_sql_remove_table return
+#' @inheritSection DBItest::spec_sql_remove_table Additional arguments
 #' @inheritSection DBItest::spec_sql_remove_table Specification
 #'
 #' @inheritParams dbGetQuery
@@ -454,3 +689,11 @@ setGeneric("dbExistsTable",
 setGeneric("dbRemoveTable",
   def = function(conn, name, ...) standardGeneric("dbRemoveTable")
 )
+
+#' @rdname hidden_aliases
+#' @export
+setMethod("dbRemoveTable", signature("DBIConnection", "Id"),
+  function(conn, name, ...) {
+    dbRemoveTable(conn, dbQuoteIdentifier(conn, name), ...)
+  }
+)
